// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include "sim.h"
#include "sun.h"
#include "dump.h"

void light_dump(struct light *in)
{
if (get_dump_status(dump_optics)==TRUE)
{
	FILE *out;
	int i;
	int ii;
	out=fopena(in->output_path,"light_Ep.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->Ep[i][ii],2.0)+pow(in->Epz[i][ii],2.0),0.5));
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(in->output_path,"light_En.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->En[i][ii],2.0)+pow(in->Enz[i][ii],2.0),0.5));
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(in->output_path,"light_E_mod.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->Ep[i][ii]+in->En[i][ii],2.0)+pow(in->Enz[i][ii]+in->Epz[i][ii],2.0),1.0));
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(in->output_path,"light_photons.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->photons[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(in->output_path,"light_photons_asb.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->photons_asb[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);


	out=fopena(in->output_path,"light_n.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->n[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);


	out=fopena(in->output_path,"light_layer.dat","w");
	for (i=0;i<in->points;i++)
	{
		fprintf(out,"%le %d\n",in->x[i],in->layer[i]);
	}
	fclose(out);

	out=fopena(in->output_path,"light_Gn.dat","w");
	for (i=0;i<in->points;i++)
	{
		fprintf(out,"%le %le\n",in->x[i]-in->device_start,in->Gn[i]);
	}
	fclose(out);

	out=fopena(in->output_path,"light_Gp.dat","w");
	for (i=0;i<in->points;i++)
	{
		fprintf(out,"%le %le\n",in->x[i],in->Gp[i]);
	}
	fclose(out);

	out=fopena(in->output_path,"light_sun.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun[i]);
	}
	fclose(out);

	out=fopena(in->output_path,"light_sun_norm.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun_norm[i]);
	}
	fclose(out);

	out=fopena(in->output_path,"light_sun_photons.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun_photons[i]);
	}
	fclose(out);

	out=fopena(in->output_path,"light_alpha.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->alpha[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);
}
}

void light_dump_1d(struct light *in, int i)
{
if (get_dump_status(dump_optics)==TRUE)
{
	FILE *out;
	int ii;
	out=fopena(in->output_path,"light_1d_Ep.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],pow(pow(in->Ep[i][ii],2.0)+pow(in->Epz[i][ii],2.0),0.5));
	}

	fclose(out);

	out=fopena(in->output_path,"light_1d_E_mod.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],pow(pow(in->Ep[i][ii]+in->En[i][ii],2.0)+pow(in->Enz[i][ii]+in->Epz[i][ii],2.0),0.5));
	}

	fclose(out);

	out=fopena(in->output_path,"light_1d_En.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],pow(pow(in->En[i][ii],2.0)+pow(in->Enz[i][ii],2.0),0.5));
	}

	fclose(out);

	out=fopena(in->output_path,"light_1d_t.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],cabs(in->t[i][ii]));
	}

	fclose(out);

	out=fopena(in->output_path,"light_1d_r.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],cabs(in->r[i][ii]));
	}

	fclose(out);

	out=fopena(in->output_path,"light_1d_photon.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],in->photons[i][ii]);
	}
	fclose(out);

	out=fopena(in->output_path,"light_1d_sun_E.dat","w");
	for (ii=0;ii<in->lpoints;ii++)
	{
		fprintf(out,"%le %le\n",in->l[ii],in->sun_E[ii]);
	}
	fclose(out);

	out=fopena(in->output_path,"light_1d_photons_asb.dat","w");
	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],in->photons_asb[i][ii]);
	}


	fclose(out);


	out=fopena(in->output_path,"light_1d_n.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],in->n[i][ii]);
	}


	fclose(out);

	out=fopena(in->output_path,"light_1d_alpha.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],in->alpha[i][ii]);
	}


	fclose(out);


	out=fopena(in->output_path,"light_1d_alpha.dat","w");

	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],in->alpha[i][ii]);
	}


	fclose(out);
	}

	double get_light_error(struct light *in)
	{
	double tot=0.0;
	int i;
	for (i=0;i<in->points;i++)
	{
		tot+=pow(pow(in->b[i],2.0)+pow(in->bz[i],2.0),0.5);
		tot+=pow(pow(in->b[i+in->points],2.0)+pow(in->bz[i+in->points],2.0),0.5);//mod(b[i+in->points]);
	}

}
}

