// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include "dump.h"
#include <math.h>

double sim_tpc_step(struct device *in)
{
//<clean>
double clamp=0.1;
double step=0.01;
double e0;
double e1;
double i0;
double i1;
double deriv;
double C=in->area*epsilon0*in->epsilonr[0]/in->ylen;
//double R1=in->Rcontact+in->Rshort;
//double R2=in->Rshunt;
solve_all(in);
i0=get_I(in);
//getchar();
double igen=(i0+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt);

e0=fabs(igen*in->Rcontact+in->Vapplied-in->tpc_externalv);
in->Vapplied+=step;
solve_all(in);
i1=get_I(in);
igen=(i1+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt);
e1=fabs(igen*in->Rcontact+in->Vapplied-in->tpc_externalv);
printf("%le\n",e1);
deriv=(e1-e0)/step;
step=-e1/deriv;
//step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
int count=0;
int max=1000;
do
{
e0=e1;
solve_all(in);
igen=(i1+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt);
e1=fabs(igen*in->Rcontact+in->Vapplied-in->tpc_externalv);
printf("%le %le %le %le\n",igen*in->Rcontact,in->Vapplied,-in->tpc_externalv,e1);
deriv=(e1-e0)/step;
step=-e1/deriv;
//double clamp=0.01;
//if (e1<clamp) clamp=e1/100.0;
//step=step/(1.0+fabs(step/clamp));
step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
//printf("ce error = %e %e\n",in->Vapplied,e1);
if (count>max) break;
count++;
}while(e1>1e-5);
//printf("result = %e %e %e %e %e\n",i1,in->Vapplied/Rshort,C*(in->Vapplied-in->Vapplied_last)/in->dt,i1+in->Vapplied/Rshort+C*(in->Vapplied-in->Vapplied_last)/in->dt);
double ret=in->Vapplied-C*(i1-in->Ilast)/in->dt;
return ret;
//</clean>
}


void sim_tpc(struct device *in)
{
//<clean>
int i;
in->dt=in->tpcdtstart;
//last_light=-1000.0;



in->go_time=FALSE;

struct istruct tpcfitout;
inter_init(&tpcfitout);


#ifndef mindump
struct istruct tpcextracted;
inter_init(&tpcextracted);

struct istruct tpcextracted2;
inter_init(&tpcextracted2);

dump_dynamic_init(in);

FILE *srhout=fopen("./srhrates.dat","w");
fclose(srhout);
#endif

printf("Running tpc. %e\n",in->tpc_externalv);
double V=in->Vstart;
int ittr=0;
//double total_steps=0;
int step=0;

solve_light(in,&(in->mylight),in->Psun,0.0);
if (fabs(in->tpc_externalv-in->Vapplied)>0.2)
{
	ramp_externalv(in,0.0,in->tpc_externalv);
}

sim_externalv(in,in->tpc_externalv);

in->tpc_power_suply=in->tpc_externalv+get_equiv_I(in)*in->Rtpc_short;

time_init(in);
device_timestep(in);

double start_field=in->Vapplied-in->Vr;

in->go_time=TRUE;
V=in->Vapplied;

in->time=-in->pulsestart*in->dt;
int tpc_running=FALSE;
double mue;
double muh;

double Psun_orig=in->Psun;

if (in->dumpitdos==TRUE)
{
	mue=get_avg_mue(in);
	muh=get_avg_muh(in);
}
carrier_count_reset(in);
render1d_reset();
//if (in->tpc_sim_all==TRUE) newton_set_min_ittr(40);
do
{
//if (step==in->pulsestart) carrier_count_reset(in);

	if (step<in->pulsestart)
	{
		solve_light(in,&(in->mylight),in->Psun,0.0);
		reset_npinit(in);
	}else
	if ((step>=in->pulsestart)&&(in->time<(in->tpclaseroff)))
	{
		solve_light(in,&(in->mylight),in->Psun,in->tpclaserpower);
		stop_start(in);
	}else
	{
		if (tpc_running==FALSE)
		{
			tpc_running=TRUE;
		}
		solve_light(in,&(in->mylight),in->Psun,0.0);
		//printf("off\n");
		stop_start(in);
	}

	in->Vapplied=V;


	if (in->tpc_sim_all==TRUE)
	{
		double i0=newton_sim_tpc_all(in);
		inter_append(&tpcfitout,in->time,i0);
		//printf("%d\n",in->tpc_sim_all);
		//getchar();
	}else
	{
		newton_sim_tpc(in);
		inter_append(&tpcfitout,in->time,get_equiv_I(in));
	}
	//sim_tpc_step(in);

	if (get_dump_status(dump_print_text)==TRUE)
	{
		printf("tpc time=%e step=%d %.1e %d ",in->time,step,in->last_error,in->last_ittr);
		printf("V=%lf current = %e mA (%e A/m^2)\n",V,get_I(in)/1e-3,get_J(in));
	}

	if (get_dump_status(dump_energy_slice_switch)==TRUE)
	{
		dump_energy_slice(in,in->ymeshpoints/2);
	}


//printf("here\n");
//printf("here\n");
	double a=0.0;
	double b=0.0;
	double c=0.0;
	double d=0.0;
	double nr1=0.0;
	double nr2=0.0;
	double nr3=0.0;
	double nr4=0.0;
//printf("here2\n");
	for (i=0;i<in->ymeshpoints;i++)
	{
		//printf("here %d %d\n",i,in->ymeshpoints);
		a+=in->nrelax[i];
		b+=in->ntrap_to_p[i];
		c+=in->prelax[i];
		d+=in->ptrap_to_n[i];

		//nr1+=in->nr1[i];
		//nr2+=in->nr2[i];
		//nr3+=in->nr3[i];
		//nr4+=in->nr4[i];

	}
//printf("here\n");
	a/=in->ymeshpoints;
	b/=in->ymeshpoints;
	c/=in->ymeshpoints;
	d/=in->ymeshpoints;

	nr1/=in->ymeshpoints;
	nr2/=in->ymeshpoints;
	nr3/=in->ymeshpoints;
	nr4/=in->ymeshpoints;
//printf("here\n");
	FILE *srhout=fopen("./srhrates.dat","a");
	//fprintf(srhout,"%le %le %le %le %le %le %le\n",in->time,a,b,c,d,get_free_n_charge(in),get_free_p_charge(in));
//	fprintf(srhout,"%le %le %le %le %le\n",in->time,nr1,nr2,nr3,nr4);
	fprintf(srhout,"%le %le %le %le %le\n",in->time,a,b,c,d);
	fclose(srhout);
//printf("here\n");


	ittr++;

	plot_now(in,in->plot_file);
	dump_dynamic_add_data(in);

	if (in->time>in->tpclaseroff) in->dt*=in->tpcdtmull;

	#ifndef mindump
	if (tpc_running==TRUE)
	{
		inter_append(&tpcextracted,in->dt,2.0*get_J(in)/in->ylen/q);
		inter_append(&tpcextracted2,in->time,2.0*get_J(in)/in->ylen/q);

	}
	#endif

	device_timestep(in);
	step++;

	if (in->time>in->tpcstop) break;

	if (in->stop==TRUE)
	{
		break;

	}


}while(1);

in->go_time=FALSE;

char outpath[200];
struct istruct tpc_heeger;

inter_sub(&tpcfitout,tpcfitout.data[tpcfitout.len-1]);
inter_copy(&tpc_heeger,&tpcfitout,TRUE);

sprintf(outpath,"%s%s",in->outputpath,"tpc_sub.dat");
inter_save(&tpcfitout,outpath);

inter_rescale(&tpcfitout,1.0, -1.0);


inter_mod(&tpcfitout);
sprintf(outpath,"%s%s",in->outputpath,"tpcfit.dat");
inter_save(&tpcfitout,outpath);


#ifndef mindump



double half_time=inter_get_quartile(&tpcfitout,0.5);


inter_mod(&tpc_heeger);
//inter_div(&tpc_heeger,in->ylen*in->ylen/(fabs(in->phi[0]-in->phi[in->ymeshpoints-1])*half_time*2.0));
sprintf(outpath,"%s%s",in->outputpath,"tpcheeger.dat");
inter_save(&tpc_heeger,outpath);

FILE *info=fopen("./tpcint.dat","w");
fprintf(info,"%le %le\n",in->tpc_externalv,inter_intergrate(&tpcfitout));
fclose(info);

inter_free(&tpc_heeger);


inter_save_a(&tpcextracted2,in->outputpath,"tpcextracted2.dat");

inter_sub(&tpcextracted,tpcextracted.data[tpcextracted.len-1]);
double tot=0.0;
for (i=0;i<tpcextracted.len;i++)
{
tpcextracted.x[i]=tpcextracted2.x[i];
tot+=tpcextracted.data[i]*tpcextracted.x[i];
tpcextracted.data[i]=tot;
}


if (in->dumpitdos==TRUE)
{


	FILE *half=fopen("./tpctransittime.dat","w");
	fprintf(half,"%le\n",half_time);
	fclose(half);

	half=fopen("./tpcmobility.dat","w");
	fprintf(half,"%le %le %le %le %le\n",mue,muh,in->mun[0],in->mup[0],(in->ylen*in->ylen)/(half_time*2.0*fabs(in->phi[0]-in->phi[in->ymeshpoints-1])));
	fclose(half);

	inter_chop(&tpcfitout,1e-9, 1.0);

	for (i=0;i<tpcfitout.len;i++)
	{
	tpcfitout.data[i]*=tpcfitout.x[i];
	tpcfitout.x[i]=((300.0*kb/q)*log(1e7*tpcfitout.x[i]));
	}

	//inter_chop(&tpcfitout,1e-9, 1.0);

	double val=inter_get_hard(&tpcfitout,0.1);

	inter_rescale(&tpcfitout,1.0, 0.1/val);

	sprintf(outpath,"%s%s",in->outputpath,"tpcitdos.dat");
	inter_save(&tpcfitout,outpath);

/*	FILE *electron=fopena(in->outputpath,"./tpc_part_ite.dat","w");
	FILE *hole=fopena(in->outputpath,"./tpc_part_ith.dat","w");

	fprintf(electron,"%le %le\n",tpcfitout.x[0],tpcfitout.data[0]);
	fprintf(hole,"%le %le\n",tpcfitout.x[0],tpcfitout.data[0]);

	for (i=1;i<tpcfitout.len;i++)
	{
		if (tpc_mue.data[i]>tpc_muh.data[i])
		{
			fprintf(electron,"%le %le\n",tpcfitout.x[i],tpcfitout.data[i]);
			fprintf(hole,"\n");
		}else
		{
			fprintf(electron,"\n");
			fprintf(hole,"%le %le\n",tpcfitout.x[i],tpcfitout.data[i]);
		}
	}

	fclose(electron);
	fclose(hole);

	for (i=0;i<tpcfitout.len;i++)
	{
	tpcfitout.data[i]=1.0/tpcfitout.data[i];
	}
	sprintf(outpath,"%s%s",in->outputpath,"tpcpreitdos.dat");
	inter_save(&tpcfitout,outpath);*/

}

dump_dynamic_save(in);
dump_dynamic_free(in);

inter_save_a(&tpcextracted,in->outputpath,"tpcextracted.dat");

inter_free(&tpcextracted);
inter_free(&tpcextracted2);

info=fopen("./tpcinfo.dat","w");
fprintf(info,"%le %le %le\n",Psun_orig,in->tpc_externalv,inter_get_hard(&tpcextracted,10e-3));
fclose(info);

#endif


inter_free(&tpcfitout);
//</clean>
}
