// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include "dump.h"
#include <math.h>

void sim_tpv(struct device *in)
{
//<clean>
int i;
int tpv_running=FALSE;
in->dt=in->tpvdtstart;


carrier_count_reset(in);

in->go_time=FALSE;

struct istruct tpvvoc;
in->Vapplied=in->tpvvoc;
inter_init(&tpvvoc);




dump_dynamic_init(in);

int step=0;




if (in->tpv_mode==tpv_mode_light)
{
printf("Running tpv. %e V\n",in->tpvvoc);
in->Psun=1000.0;
solve_light(in,&(in->mylight),in->Psun,0.0);
set_light_for_voc(in,in->tpvvoc);
in->Vapplied=in->tpvvoc;
}else
{
printf("Running tpv. %e W\n",in->Psun);
solve_light(in,&(in->mylight),in->Psun,0.0);
newton_sim_tpv(in);
}

if (in->simwire==TRUE)
{
	wire_init(in->Vapplied,1e-9);
}


//sim_externalv(in,in->in->tpvvoc);
newton_sim_tpv_fast(in,FALSE);
in->ilast=get_I(in)+in->Vapplied/in->Rshunt;
in->Vapplied_last=in->Vapplied;

time_init(in);
device_timestep(in);
in->dt=in->tpvdtstart;
in->go_time=TRUE;
in->time=in->tpvstart;

newton_sim_tpv_fast(in,TRUE);

FILE *out=fopen("./tpvpsun.dat","w");
fprintf(out,"%le\n",in->Psun);
fclose(out);

FILE *tpvn=fopen("./tpv_cden.dat","w");
fprintf(tpvn,"%le\n",get_extracted_np(in));
fclose(tpvn);



do
{
	if (in->time<0.0)
	{
		solve_light(in,&(in->mylight),in->Psun,0.0);
		reset_npinit(in);
	}else
	if ((in->time>=0.0)&&(in->time<(in->tpvlaseroff)))
	{
		solve_light(in,&(in->mylight),in->Psun,in->tpvlaserpower);
		int ii;

		if (get_dump_status(dump_iodump)==TRUE)
		{
			FILE *out =fopena(in->outputpath,"./Gn_TPV_laser.dat","w");
			for (ii=0;ii<in->ymeshpoints;ii++)
			{
					fprintf(out,"%e %le\n",in->ymesh[ii],in->Gn[ii]);
			}
			fclose(out);
		}
	}else
	{
		if (tpv_running==FALSE)
		{
			//
			tpv_running=TRUE;
		}
		solve_light(in,&(in->mylight),in->Psun,0.0);
	}

	//solve_all(in);
	if (in->simwire==FALSE)
	{
		newton_sim_tpv_fast(in,TRUE);

	}else
	{
		wire_set_dt(in->dt);
		newton_sim_tpv_wire(in);
		wire_addtime();
	}

	if (get_dump_status(dump_print_text)==TRUE)
	{
	printf("tpv time=%e step=%d %.1e ",in->time,step,in->last_error);
	char out[100];
	sprintf(out,"V=%lf current = %e mA (%e A/m^2)\n",in->Vapplied,get_I(in)/1e-3,get_J(in));

	printdebug(out);
	}


	if (tpv_running==TRUE)
	{

		dump_dynamic_add_data(in);
	}

	if (in->simwire==FALSE)
	{
		inter_append(&tpvvoc,in->time,get_V_TPV(in));
	}else
	{
		inter_append(&tpvvoc,in->time,wire_get_voltage_at_end());
	}

	plot_now(in,"plot");

	if (in->time>in->tpvlaseroff) in->dt*=in->tpvdtmull;

	if (step==in->tpvpulsestart)
	{
		in->dt=in->tpvdtstart;
	}

	step++;
	device_timestep(in);



	if (in->time>in->tpvstop) break;

	if (in->stop==TRUE)
	{
		break;

	}

in->ilast=get_I(in)+in->C*(in->Vapplied-in->Vapplied_last)/in->dt+in->Vapplied/in->Rshunt;

}while(1);

in->go_time=FALSE;

char outpath[200];



sprintf(outpath,"%s%s",in->outputpath,"tpv_voc.dat");
inter_save(&tpvvoc,outpath);

inter_sub(&tpvvoc,tpvvoc.data[0]);
sprintf(outpath,"%s%s",in->outputpath,"tpv_voc_norm.dat");
inter_save(&tpvvoc,outpath);

inter_free(&tpvvoc);

dump_dynamic_save(in);
dump_dynamic_free(in);
//</clean>
}

